# This 'executable' file intentionally contains only BTF sections
{{package}}: statically-linked-binary [usr/src/*/vmlinux]
{{package}}: unstripped-binary-or-object [usr/src/*/vmlinux]
{% if arch in ['arm64'] %}
{{package}}: shared-library-lacks-prerequisites [usr/src/*/vmlinux]
{% endif %}
