.config .kernel* Module.symvers include  usr/src/{{package}}
scripts/module.lds                       usr/src/{{package}}/arch/{{kernel_arch}}
arch/{{kernel_arch}}/include             usr/src/{{package}}/arch/{{kernel_arch}}
{% if arch == 'powerpc' %}
arch/{{kernel_arch}}/lib/crtsavres.o     usr/src/{{package}}/arch/{{kernel_arch}}/lib
{% endif %}
