lib/modules/{{abiname}}{{localversion}}/kernel      usr/lib/debug/lib/modules/{{abiname}}{{localversion}}
lib/modules/{{abiname}}{{localversion}}/System.map  usr/lib/debug/lib/modules/{{abiname}}{{localversion}}
lib/modules/{{abiname}}{{localversion}}/vdso        usr/lib/debug/lib/modules/{{abiname}}{{localversion}}
lib/modules/{{abiname}}{{localversion}}/vmlinux     usr/lib/debug/lib/modules/{{abiname}}{{localversion}}
