# linux-signed-* source packages are generated by the linux source
# package, so it is OK for their binaries to share documentation
{{package}}: usr-share-doc-symlink-to-foreign-package linux-base-{{abiname}}{{localversion}}
